package com.outr.webcommunicator.netty.communicator

import com.outr.webcommunicator.{CommunicatorManager, Communicator}
import com.outr.webcommunicator.netty.{URIWebResource, NettyWebapp}
import org.jboss.netty.channel.{MessageEvent, ChannelHandlerContext}
import org.powerscala.concurrent.Executor
import org.jboss.netty.handler.codec.http.websocketx.WebSocketFrame

/**
 * @author Matt Hicks <mhicks@powerscala.org>
 */
trait NettyCommunicatorManager[C <: Communicator] extends NettyWebapp with CommunicatorManager[C] {
  def webSocketEnabled = true
  def webSocketUri = "/websocket"
  def ajaxPollingUri = "/ajax/polling"
  def ajaxReceiverUri = "/ajax/receiver"

  // Register the websocket handler
  if (webSocketEnabled) {
    register(new URIWebResource(webSocketUri, () => new CommunicatorWebSocketHandler[C](this)) {
      override def request(webapp: NettyWebapp, context: ChannelHandlerContext, event: MessageEvent) = event.getMessage match {
        case frame: WebSocketFrame => Some(f())
        case _ => super.request(webapp, context, event)
      }
    })
  }
  // Register the polling handler
  register(URIWebResource(ajaxPollingUri) {
    new CommunicatorAJAXPollingHandler(this)
  })
  // Register the AJAX receiver
  register(URIWebResource(ajaxReceiverUri) {
    new CommunicatorAJAXReceivingHandler(this)
  })

  val updater = Executor.scheduleWithFixedDelay(5.0, 5.0) {   // Periodically clear out old Communicator instances that are dead
    update()
  }
}
