package com.outr.webcommunicator

import org.jboss.netty.handler.codec.http.{HttpMethod, QueryStringDecoder, HttpRequest}
import com.google.common.net.HttpHeaders

import scala.collection.JavaConversions._

/**
 * @author Matt Hicks <matt@outr.com>
 */
package object netty {
  implicit def request2URL(request: HttpRequest) = {
    val decoder = new QueryStringDecoder(request.getUri)
    val hostAndPort = request.getHeader(HttpHeaders.HOST) match {
      case null => "unknown"
      case h => h
    }
    val hpSeparator = hostAndPort.indexOf(':')
    val host = if (hpSeparator != -1) {
      hostAndPort.substring(0, hpSeparator)
    } else {
      hostAndPort
    }
    val port = if (hpSeparator != -1) {
      hostAndPort.substring(hpSeparator + 1).toInt
    } else {
      80
    }
    val parameters = decoder.getParameters.map {
      case (key, values) => key -> values.toList
    }.toMap
    NettyWebapp.createURL(method = httpMethod2Method(request.getMethod),
        protocol = "http",      // TODO: support other protocols
        host = host,
        port = port,
        path = decoder.getPath,
        parameters = parameters)
  }

  implicit def httpMethod2Method(m: HttpMethod) = Method.values.find(method => method.value == m.getName.toLowerCase).getOrElse(null)
}
