package com.outr.webcommunicator.proxy

import org.jboss.netty.handler.codec.http.HttpRequest
import com.outr.webcommunicator.URL
import com.outr.webcommunicator.netty._
import org.powerscala.Priority

/**
 * @author Matt Hicks <mhicks@outr.com>
 */
trait URLProxy extends HttpRequestProxy {
  def apply(url: URL): Option[(String, Int)]

  def apply(request: HttpRequest): Option[(String, Int)] = apply(request2URL(request))
}

object URLProxy {
  def create(matcher: URL => Boolean, host: String, port: Int, priority: Priority = Priority.Normal): URLProxy = {
    val p = priority
    new URLProxy {
      override def priority = p

      def apply(url: URL): Option[(String, Int)] = if (matcher(url)) {
        Some(host, port)
      } else {
        None
      }
    }
  }

  def apply(matcher: String => Boolean, host: String, port: Int, priority: Priority = Priority.Normal): URLProxy = {
    create((url: URL) => matcher(url.toString), host, port, priority)
  }
}